#include "common.h"
#include <pthread.h>
#include <wait.h>
#include <signal.h>

#define MAX_PARM 100
int execute( int m, char **argv ) {
   char msg[ 80 ] = "execute command: ";
   int i, bp = 0;     
   char *argp[ MAX_PARM ];
   for( i = 0; i < MAX_PARM; i++ ) argp[ i ] = NULL;
   for( i = 0; i < m; i++ ) {
      char *p = argv[ i ];
      argp[ bp++ ] = p;
      p = strtok( p, " " );
      if( NULL == p ) continue;
      while( NULL != ( p = strtok( NULL, " " ) ) ) {
         argp[ bp++ ] = p;
      } 
   }
   for( i = 0; i < bp; i++ ) 
      sprintf( msg + strlen( msg ), "<%s> ", argp[ i ] );
   debug_msg( msg, 0 );
   return execvp( argp[ 0 ], argp );
}

void set_rt( int policy ) {                // установить тип диспетчирования
   struct sched_param param;
   char msg[ 80 ] = "set scheduling ";
   pthread_t tid = pthread_self();
   param.sched_priority = sched_get_priority_max( policy );
   if( pthread_setschedparam( tid, policy, &param ) != 0 )
      perror( "pthread_setschedparam" ), exit( EXIT_FAILURE );
   if( pthread_getschedparam( tid, &policy, &param ) != 0 )
      perror( "pthread_getschedparam" ), exit( EXIT_FAILURE );
   sprintf( msg + strlen( msg ), "%s with priority=%d",
            0 == policy ? "OTHER" : 1 == policy ? "FIFO" : 2 == policy ? "RR" : "UNKNOWN",
            param.sched_priority );
   debug_msg( msg, 0 );
};

int main( int argc, char **argv, char **envp ) {
   int c, secrun = 5, signo = SIGQUIT, policy = SCHED_RR;
   char sopt[] = "t:s:p:v";
   while( -1 != ( c = getopt( argc, argv, sopt ) ) )
      switch( c ) {
         case 't':              // продолжительность выполнения (сек.)
            secrun = atoi( optarg );
            break;
         case 's':              // сигнал завершения
            signo = atoi( optarg );
            break;
         case 'p':              // политика планирования
            policy = optarg[ 0 ] == 'R' ? SCHED_RR :
                     optarg[ 0 ] == 'F' ? SCHED_FIFO :
                     SCHED_OTHER;
            break;
         case 'v':
            debug_level++;
            break;
         default :
            printf( "option must be: %s\n", sopt );
            return( EXIT_FAILURE );
      }
   debug_msg( "main started", 1 );
   pid_t pid = fork();          // процесс разветвился
   if( pid == -1 ) perror( "fork" ), exit( EXIT_FAILURE );
   if( pid > 0 ) {              // родитель
      int status;
      sleep( secrun );
      status = waitpid( -1, NULL, WNOHANG );
      if( status <= 0 ) {
         char msg[ 80 ];
         sprintf( msg, "send signal %d to child", signo );
         debug_msg( msg, 1 );
         kill( pid, signo );
         waitpid( pid, NULL, 0 );
      }
      debug_msg( "main finished", 1 );
      exit( EXIT_SUCCESS );
   };
   if( pid == 0 ) {             // потомок
      set_rt( policy );
      int m = argc - optind;
      if( m > 0 ) {
         if( -1 == execute( m, argv + optind ) ) perror( "exec" );
      }   
      else {
         void handler( int signo ) {
            char msg[ 80 ];
            sprintf( msg, "received signal %d from parent", signo );
            debug_msg( msg, 1 );
            exit( EXIT_SUCCESS );
         };
         signal( signo, handler );
         debug_msg( "child loop started", 1 );
         while( 1 ) {};
      }
      debug_msg( "child finished", 1 );
      exit( EXIT_SUCCESS );
   }
   return( EXIT_SUCCESS );
}

